package com.ti.mobpo.ui.viewmodels

import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.ti.mobpo.data.Favourite
import com.ti.mobpo.data.FavouritesRepository
import com.ti.mobpo.model.FavouriteUiState
import com.ti.mobpo.model.PokemonDetails
import com.ti.mobpo.network.PokeApi
import com.ti.mobpo.ui.util.FeatureManager
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import java.io.IOException

class FavouritesViewModel(private val favouritesRepository: FavouritesRepository,
                          private val featureManager: FeatureManager
) : ViewModel() {

    private val service = PokeApi.retrofitService
    private val _pokemonDetails = MutableStateFlow<List<PokemonDetails>?>(null)
    val pokemonDetails: StateFlow<List<PokemonDetails>?> = _pokemonDetails.asStateFlow()
    val accessCheckFailed: MutableState<Boolean> = mutableStateOf(false)

    fun toggleDebugMode() {
        featureManager.setPaidFeatureEnabled(!featureManager.hasAccessToPaidFeature())
    }
    fun loadFavourites() {
        // featureManager.setPaidFeatureEnabled(false) enable and disable acccess
        // Ugly workaround to make sure all the favourites are loaded before displaying them
        viewModelScope.launch {
            if(!featureManager.hasAccessToPaidFeature()) {
                accessCheckFailed.value = true
                return@launch
            }
            try {
                val favouritesList: StateFlow<FavouriteUiState> =
                    favouritesRepository.getAllItems().map { FavouriteUiState(it) }
                        .stateIn(
                            scope = viewModelScope,
                            started = SharingStarted.WhileSubscribed(5_000L),
                            initialValue = FavouriteUiState()
                        )
                favouritesList.collect { state ->
                    val detailsList = mutableListOf<PokemonDetails>()
                    for (favourite in state.favourites) {
                        val details = service.getPokemonDetails(favourite.id)
                        detailsList.add(details.copy(isFavorite = true))
                    }
                    _pokemonDetails.value = detailsList
                }
            } catch (e: IOException) {
                /* Handle error */
            }
        }
    }

    fun toggleFavorite(pokemonId: Int) {
        _pokemonDetails.value = _pokemonDetails.value?.map { pokemon ->
            if (pokemon.id == pokemonId) {
                pokemon.copy(isFavorite = !pokemon.isFavorite)
            } else {
                pokemon
            }
        }
        viewModelScope.launch {
            if (_pokemonDetails.value != null) {
                val pokemon = _pokemonDetails.value!!.find { it.id == pokemonId }
                pokemon?.let {
                    if (it.isFavorite) {
                        favouritesRepository.insertItem(Favourite(it.id, it.name))
                    } else {
                        favouritesRepository.deleteItem(Favourite(it.id, it.name))
                    }
                }
            }
        }
    }
}
